#include "MnnSettings.hpp"

#include <stdexcept>

using namespace FJ;
using namespace std;

/*
{
"input_files" : 
[
   { 
   "name" : "zzz_input_name"
   }
],
"settings" :
{
   "approximate" : zzz_approximate,
   "automatic_order" : zzz_automatic_order,
   "num_mads" : zzz_num_mads,
   "num_neighbors" : zzz_num_neighbors,
   "robust_iterations" : zzz_robust_iterations,
   "robust_trim" : zzz_robust_trim,
   "run_ID" : "zzz_run_ID",
   "reference_policy" : "zzz_reference_policy"
},
"summary_file" : "zzz_summary_file",
"output_files" :
[
   "zzz_output_file"
]
}
*/
void MnnSettings::setDefaults()
{
   mApproximate = Defaults::approximate;

   mAutomaticOrder = Defaults::automatic_order;

   mMassCap = Defaults::mass_cap;

   mNumMads = Defaults::num_mads;

   mNumNeighbors = Defaults::num_neighbors;

   mNumThreads = Defaults::num_threads;

   mnncorrect::ReferencePolicy mReferencePolicy = Defaults::reference_policy;

   mNumRobustIterations = Defaults::robust_iterations;

   mRobustTrim = Defaults::robust_trim;

   mRunID = Defaults::run_ID;
}

void MnnSettings::processLoadedJSON()
{
/*
{
   "input_files" : 
   [
      { 
      "name" : "develop 2 batches.csv",
      "is_normalized" : true
      }
   ],
   "settings" :
   {
      "approximate" : true,
      "automatic_order" : false,
      "mass_cap" : 1000000,
      "num_mads" : 4,
      "num_neighbors" : 11,
      "robust_iterations" : 3,
      "robust_trim" : 0.3,
      "run_ID" : "RUN42",
      "reference_policy" : "input"
   },
   "summary_file" : "mnn_summary.json",
   "output_files" :
   [
      "mnn_corrected.csv"
   ]
}
*/

   // validate loaded data
   if( numInputFiles() != 1 )
      throw runtime_error( "There must be exactly one input file" );

   if( summaryFile(false).empty() )
      throw runtime_error( "No summary file" );

   if( numOutputFiles() != 1 )
      throw runtime_error( "There must be exactly one output file" );

   // set all data to defaults
   setDefaults();

   if( has_setting( approximate_Att ) )
      mApproximate = setting( approximate_Att ).getBool();

   if( has_setting( automatic_order_Att ) )
      mAutomaticOrder = setting( automatic_order_Att ).getBool();

   //if( has_setting( mass_cap_Att ) )
   //   mMaxObservations = setting( mass_cap_Att ).getInt();

   if( has_setting( num_mads_Att ) )
      mNumMads = setting( num_mads_Att ).getDouble();

   if( has_setting( num_neighbors_Att ) )
      mNumNeighbors = setting( num_neighbors_Att ).getInt();

   if( has_setting( reference_policy_Att ) )
   {
      const string text { setting( reference_policy_Att ).getString() };
      if( text == input_Val )
         mReferencePolicy = mnncorrect::ReferencePolicy::Input;
      else if( text == max_size_Val )
         mReferencePolicy = mnncorrect::ReferencePolicy::MaxSize;
      else if( text == max_variance_Val )
         mReferencePolicy = mnncorrect::ReferencePolicy::MaxVariance;
      else if( text == max_Rss_Val )
         mReferencePolicy = mnncorrect::ReferencePolicy::MaxRss;
      else
      {
         string temp { text };
         temp += " is not a valid reference policy";
         throw runtime_error( temp );
      }
   }

   if( has_setting( robust_iterations_Att ) )
      mNumRobustIterations = setting( robust_iterations_Att ).getInt();

   if( has_setting( robust_trim_Att ) )
      mRobustTrim = setting( robust_trim_Att ).getDouble();

   // this is the only required setting
   if( has_setting( run_ID_Att ) )
      mRunID = setting( run_ID_Att ).getString();
   else
      throw runtime_error( "No run ID specified" );
}

MnnSettings::MnnSettings()
   : Settings()
{}

MnnSettings::MnnSettings( const std::string& JSONSetupFile )
   : Settings()
{
   loadJSONSetup( JSONSetupFile );
}
