#ifndef MNN_SETTINGS_HPP
#define MNN_SETTINGS_HPP

#include "ReferencePolicy.hpp"
#include "Settings.hpp"

namespace FJ
{
   class MnnSettings : public Settings
   {
   public:

      MnnSettings();
      MnnSettings( const std::string& JSONSetupFile );

      MnnSettings( const MnnSettings& source ) = default;
      MnnSettings& operator=( const MnnSettings& rhs ) = default;
      MnnSettings( MnnSettings&& source ) = default;
      MnnSettings& operator=( MnnSettings&& rhs ) = default;

      // getters
      bool approximate() const { return mApproximate; }

      bool automaticOrder() const { return mAutomaticOrder; }

      std::size_t massCap() const { return mMassCap;}

      double numMads() const { return mNumMads; }

      int numNeighbors() const { return mNumNeighbors; }

      int numRobustInterations() const { return mNumRobustIterations; }

      int numThreads() const { return mNumThreads; }

      mnncorrect::ReferencePolicy referencePolicy() const { return mReferencePolicy; }

      double robustTrim() const { return mRobustTrim; }

      std::string runID() const { return mRunID; }

   protected:

      void processLoadedJSON() override;

   private:

      void setDefaults();

      inline static const char* approximate_Att { "approximate" };
      inline static const char* automatic_order_Att { "automatic_order" };
      //inline static const char* mass_cap_Att { "mass_cap" };
      inline static const char* num_mads_Att { "num_mads" };
      inline static const char* num_neighbors_Att { "num_neighbors" };
      inline static const char* robust_iterations_Att { "robust_iterations" };
      inline static const char* reference_policy_Att { "reference_policy" };
      inline static const char* robust_trim_Att { "robust_trim" };
      inline static const char* run_ID_Att { "run_ID" };

      // JSON names for reference policy settings
      inline static const char* input_Val { "Input" };
      inline static const char* max_Rss_Val { "MaxRSS" };
      inline static const char* max_size_Val { "MaxSize" };
      inline static const char* max_variance_Val { "MaxVariance" };

      // Defaults copied from MnnCorrect.hpp
      struct Defaults
      {
         static constexpr bool approximate = false;

         static constexpr bool automatic_order = true;

         static constexpr std::size_t mass_cap = -1;

         static constexpr double num_mads = 3.0;

         static constexpr int num_neighbors = 15;

         static constexpr int num_threads = 10; // code ignores and uses 1/2 max number of threads

         static constexpr int robust_iterations = 2;

         static constexpr double robust_trim = 0.25;

         static constexpr mnncorrect::ReferencePolicy reference_policy { mnncorrect::MaxRss };

         static constexpr const char* run_ID = "NO_RUN_ID"; // not from MnnCorrect.cpp 
      };

      bool mApproximate { Defaults::approximate };

      bool mAutomaticOrder { Defaults::automatic_order };

      std::size_t mMassCap { Defaults::mass_cap };

      double mNumMads { Defaults::num_mads };

      int mNumNeighbors { Defaults::num_neighbors };

      int mNumThreads { Defaults::num_threads };

      mnncorrect::ReferencePolicy mReferencePolicy { Defaults::reference_policy };

      int mNumRobustIterations { Defaults::robust_iterations };

      double mRobustTrim { Defaults::robust_trim };

      std::string mRunID { Defaults::run_ID }; // not from MnnCorrect.cpp

   };
}
#endif