//#include "fast_clustering.hpp"
#include "run_mnn.hpp"
#include "MnnSettings.hpp"
#include "utilities.hpp"

#include "Summary.hpp"
#include <chrono>
#include <exception>
#include <iostream>
#include <sstream>

using namespace FJ;
using namespace std;

int main(int argc, char *argv[])
{
   string settingsFile;
   MnnSettings settings;
   Summary summary;
   try
   {
      if( argc == 2 )
      {
         settingsFile = argv[1];
         settingsFile = add_file_path( settingsFile );
      }
      else
         throw runtime_error( "The C++ program must be called with exactly one command-line argument" );

      // ***** load setup *****
      cout << "Before loading setting" << endl;
      settings.loadJSONSetup( settingsFile );
      cout << "After loading settings" << endl;

      cout << "Input file is " << settings.inputFile(0).name << endl
         << "Number of neighbors is " << settings.numNeighbors() << endl;

      cout << "Before running MNN" << endl;
      auto start = chrono::steady_clock::now();

      // ***** user calculations *****
      summary = run_mnn( settings );
      // *******************************

      auto end = chrono::steady_clock::now();
      chrono::duration<double> elapsed_seconds = end-start;
      cout << "After running MNN" << endl;

      cout << "MNN batch correction on " << summary.numEvents()
         << " events took " << elapsed_seconds.count() << " seconds";
   }
   catch( exception& error)
   {
      cout << "Error: " << error.what() << endl;
      summary.is_success( false );
      summary.information( error.what() );
   }
   catch(...)
   {
      cout << "Unknown error" << endl;
      summary.is_success( false );
      summary.information( "Unknown error" );
   }

   // if we can't save the summary file there's nothing more to do. The
   // caller must recognize that the absence of the summary file means
   // that it couldn't be written and should tell the plugin user that
   // there was an unknown error
   // ***** save summary *****
   summary.saveAsINI( settings.summaryFile() );
}
