/*
 * Exceptions.cpp
 *
 *  Created on: Nov 21, 2021
 *      Author: shahids
 */

#include "easylogging++.h"
#include "Exceptions.h"

ExceptionType::Identifier ExceptionType :: FromString(std::string identifier)
{
	if(identifier == "logical_error")
		return logical_error;
	if(identifier == "domain_error")
		return domain_error;
	if(identifier == "invalid_argument")
		return invalid_argument;
	if(identifier == "length_error")
		return length_error;
	if(identifier == "out_of_range")
		return out_of_range;
	if(identifier == "overflow_error")
		return overflow_error;
	if(identifier == "range_error")
		return range_error;
	if(identifier == "underflow_error")
		return underflow_error;
	if(identifier == "bad_alloc")
		return bad_alloc;
	if(identifier == "bad_cast")
		return bad_cast;
	if(identifier == "bad_typeid")
		return bad_typeid;
	if(identifier == "bad_exception")
		return bad_exception;
	if(identifier == "logic_failure")
		return logic_failure;
	if(identifier == "runtime_error")
		return runtime_error;
	if(identifier == "future_error")
		return future_error;
	if(identifier == "non_existent_local_time")
		return non_existent_local_time;
	if(identifier == "ambiguous_local_time")
		return ambiguous_local_time;
	if(identifier == "bad_any_cast")
		return bad_any_cast;
	if(identifier == "bad_array_new_length")
		return bad_array_new_length;
	if(identifier == "bad_variant_access")
		return bad_variant_access;
	if(identifier == "system_error")
		return system_error;
	if(identifier == "failure")
		return failure;
	if(identifier == "format_error")
		return format_error;
	if(identifier == "filesystem_error")
		return filesystem_error;
	if(identifier == "bad_optional_access")
		return bad_optional_access;
	if(identifier == "regex_error")
		return regex_error;
	if(identifier == "tx_exception")
		return tx_exception;
	if(identifier == "bad_weak_ptr")
		return bad_weak_ptr;
	if(identifier == "bad_function_call")
		return bad_function_call;
	if(identifier == "catch_all")
		return catch_all;
	if(identifier == "F1Exception")
		return F1Exception;
	if(identifier == "ParseError")
		return ParseError;
	if(identifier == "NumericalException")
	return NumericalException;
	if(identifier == "FCSReadingException")
	   return FCSReadingException;
	if(identifier == "disallowed_use_case")
		return disallowed_use_case;
	if(identifier == "AnalysisAlreadyExists")
		return AnalysisAlreadyExists;
	if(identifier == "FormulaError")
		return FormulaError;
	if(identifier == "object_missing")
		return object_missing;
	if(identifier == "unexpected_modification")
		return unexpected_modification;
	if(identifier == "too_few_events")
		return too_few_events;
	return exception;
}

std::string ExceptionType :: ToString(ExceptionType::Identifier identifier)
{
	switch(identifier)
	{
		case logical_error: return "logical_error";
		case domain_error: return "domain_error";
		case invalid_argument: return "invalid_argument";
		case length_error: return "length_error";
		case out_of_range: return "out_of_range";
		case overflow_error: return "overflow_error";
		case range_error: return "range_error";
		case underflow_error: return "underflow_error";
		case bad_alloc: return "bad_alloc";
		case bad_cast: return "bad_cast";
		case bad_typeid: return "bad_typeid";
		case bad_exception: return "bad_exception";
		case logic_failure: return "logic_failure";
		case runtime_error: return "runtime_error";
		case exception: return "exception";
		case future_error: return "future_error";
		case non_existent_local_time: return "non_existent_local_time";
		case ambiguous_local_time: return "ambiguous_local_time";
		case bad_any_cast: return "bad_any_cast";
		case bad_array_new_length: return "bad_array_new_length";
		case bad_variant_access: return "bad_variant_access";
		case failure: return "failure";
		case format_error: return "format_error";
		case filesystem_error: return "filesystem_error";
		case bad_optional_access: return "bad_optional_access";
		case regex_error: return "regex_error";
		case tx_exception: return "tx_exception";
		case bad_weak_ptr: return "bad_weak_ptr";
		case bad_function_call: return "bad_function_call";
		case F1Exception: return "F1Exception";
		case ParseError: return "ParseError";
		case CancellationException: return "CancellationException";
		case NumericalException: return "NumericalException";
		case FCSReadingException: return "FCSReadingException";
		case disallowed_use_case: return "disallowed_use_case";
		case AnalysisAlreadyExists: return "AnalysisAlreadyExists";
		case FormulaError: return "FormulaError";
		case object_missing: return "object_missing";
		case too_few_events: return "too_few_events";
		case unexpected_modification: return "unexpected_modification";
		case catch_all: return "catch_all";
		default: return "UNKNOWN ERROR";
	}
}

const std::string EngineException :: exceptionDetails() const
{
	std::string returnMessage = what();
	std::string logStatement = std::string();

	switch(fIdentifier)
	{
		//"->" referring to the subclass
		//Logical Errors
		case ExceptionType::Identifier::logical_error:
		/* exception->logical_error
		 Defines a type of object to be thrown as exception. It reports errors that are a consequence
		 of faulty logic within the program such as violating logical preconditions or class invariants and may be preventable.
		 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::logical_error);
			break;
			case ExceptionType::Identifier::domain_error :
			/* exception->logical_error->domain_error
			This is an exception thrown when a mathematically invalid domain is used.
			If function is not define for an input, we throw this exception.
			Defines a type of object to be thrown as exception. It may be used by the implementation to report domain errors,
			that is, situations where the inputs are outside of the domain on which an operation is defined.
			The standard library components do not throw this exception (mathematical functions report domain errors as specified in math_errhandling).
			Third-party libraries, however, use this. For example, boost.math throws std::domain_error if boost::math::policies::throw_on_error is enabled (the default setting).
			We can recover from this exception and send an error to the client by explaining the we didn't provide a valid input to the calculation.
			*/
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::domain_error);
			break;
			case ExceptionType::Identifier::invalid_argument:
			/* exception->logical_error->invalid_argument
			Defines a type of object to be thrown as exception.
			It reports errors that arise because an argument value has not been accepted.
			*/
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::invalid_argument);
			break;
			case ExceptionType::Identifier::length_error:
			/* exception->logical_error->length_error
			 Defines a type of object to be thrown as exception.
			 It reports errors that result from attempts to exceed implementation defined length limits for some object..
			 This is thrown when a too big std::string is created.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::length_error);
			break;
			case ExceptionType::Identifier::out_of_range:
			/* exception->logical_error->out_of_range
			 Defines a type of object to be thrown as exception.
			 It reports errors that are consequence of attempt to access elements out of defined range.
			 This can be thrown by the 'at' method, for example a std::vector and std::bitset<>::operator[]().
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::out_of_range);
			break;
			case ExceptionType::Identifier::future_error:
			/* exception->logical_error->future_error
			 Introduced in C++11. The class std::future_error defines an exception object that is thrown on failure by
			 the functions in the thread library that deal with asynchronous execution and shared states (std::future, std::promise, etc).
			 Similar to std::system_error, this exception carries an error code compatible with std::error_code.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::future_error);
			break;
		  case ExceptionType::Identifier::bad_optional_access:
			/* exception->bad_optional_access
			 Introduced in C++17.
			 Defines a type of object to be thrown by std::optional::value when accessing an optional object that does not contain a value.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_optional_access);
			break;
		//Runtime Errors
		case ExceptionType::Identifier::runtime_error:
		/* exception->runtime_error
		 Defines a type of object to be thrown as exception. It reports errors that are due to events beyond the scope of the program and can not be easily predicted.
		 Exceptions of type std::runtime_error are thrown by the following standard library components: std::locale::locale and std::locale::combine.
		 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::runtime_error);
		break;
			case ExceptionType::Identifier::overflow_error:
			/* exception->runtime_error->overflow_error
			 This is thrown if a mathematical overflow occurs.
			 Defines a type of object to be thrown as exception. It can be used to report arithmetic overflow
			 errors (that is, situations where a result of a computation is too large for the destination type)
			 The only standard library components that throw this exception are std::bitset::to_ulong and std::bitset::to_ullong.
			 The mathematical functions of the standard library components do not throw this exception
			 (mathematical functions report overflow errors as specified in math_errhandling). Third-party libraries, however, use this. For example, boost.math throws std::overflow_error if boost::math::policies::throw_on_error is enabled (the default setting).
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::overflow_error);
			break;
			case ExceptionType::Identifier::range_error:
			/* exception->runtime_error->range_error
			 This is occurred when you try to store a value which is out of range.
			 Defines a type of object to be thrown as exception.
			 It can be used to report range errors (that is, situations where a result of a computation cannot be represented by the destination type).
			 The only standard library components that throw this exception are std::wstring_convert::from_bytes and std::wstring_convert::to_bytes.
			 The mathematical functions in the standard library components do not throw this exception
			 (mathematical functions report range errors as specified in math_errhandling).
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::range_error);
			break;
			case ExceptionType::Identifier::underflow_error:
			/* exception->runtime_error->underflow_error
			 This is thrown if a mathematical underflow occurs.
			 Defines a type of object to be thrown as exception. It may be used to report arithmetic underflow
			 errors (that is, situations where the result of a computation is a subnormal floating-point value).
			 The standard library components do not throw this exception (mathematical functions report underflow
			 errors as specified in math_errhandling). Third-party libraries, however, use this.
			 For example, boost.math throws std::underflow_error if boost::math::policies::throw_on_error is enabled (the default setting).
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::underflow_error);
			break;
			case ExceptionType::Identifier::regex_error:
			/* exception->runtime_error->regex_error
			 Introduced in C++11
			 Defines the type of exception object thrown to report errors in the regular expressions library.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::regex_error);
			break;
			case ExceptionType::Identifier::system_error:
			/* exception->runtime_error->system_error
			 std::system_error is the type of the exception thrown by various library
			 functions (typically the functions that interface with the OS facilities, e.g. the constructor of std::thread)
			 when the exception has an associated std::error_code, which may be reported.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::system_error);
			break;
			//system_error
				case ExceptionType::Identifier::failure:
				/* since C++11 : exception->runtime_error->system_error->ios_base::failure
				   before C++11 : exception->ios_base::failure
				 The class std::ios_base::failure defines an exception object that is thrown on failure by the functions in the Input/Output library.
				 */
				logStatement = ExceptionType::ToString(ExceptionType::Identifier::failure);
				break;
				case ExceptionType::Identifier::filesystem_error:
				/* exception->runtime_error->system_error->std::filesystem::filesystem_error
				 Introduced in C++17.
				 The class std::filesystem::filesystem_error defines an exception object that is thrown on failure
				 by the throwing overloads of the functions in the filesystem library.
				 */
				logStatement = ExceptionType::ToString(ExceptionType::Identifier::filesystem_error);
				break;
			case ExceptionType::Identifier::tx_exception:
			/* exception->runtime_error->tx_exception
			 Defines an exception type that can be used to cancel and roll back an atomic transaction initiated by the keyword atomic_cancel
			 If T is not TriviallyCopyable, the program that specializes std::tx_exception<T> is ill-formed.
			*/
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::tx_exception);
			break;
			case ExceptionType::Identifier::non_existent_local_time:
			/* exception->runtime_error->non_existent_local_time
			 Introduced in C++20
			 Defines a type of object to be thrown as exception to report that an attempt was made to convert
			 a nonexistent std::chrono::local_time to a std::chrono::sys_time without specifying
			 a std::chrono::choose (such as choose::earliest or choose::latest). This exception is
			 thrown by std::chrono::time_zone::to_sys and functions that call it (such as the constructors
			 of std::chrono::zoned_time that takes a std::chrono::local_time).
			*/
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::non_existent_local_time);
			case ExceptionType::Identifier::ambiguous_local_time:
			/* exception->runtime_error->ambiguous_local_time
			 Introduced in C++20
			 Defines a type of object to be thrown as exception to report that an attempt was made
			 to convert a ambiguous std::chrono::local_time to a std::chrono::sys_time without specifying
			 a std::chrono::choose (such as choose::earliest or choose::latest). This exception is
			 thrown by std::chrono::time_zone::to_sys and functions that call it (such as the
			 constructors of std::chrono::zoned_time that takes a std::chrono::local_time).
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::ambiguous_local_time);
			break;
			case ExceptionType::Identifier::format_error:
			/* exception->runtime_error->format_error
			 Introduced in C++20
			 Defines the type of exception object thrown to report errors in the formatting library.
			*/
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::format_error);
			break;
		  case ExceptionType::Identifier::bad_typeid:
			/* exception->bad_typeid
			 An exception of this type is thrown when a typeid operator is applied to
			 a dereferenced null pointer value of a polymorphic type.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_typeid);
		  break;
		  case ExceptionType::Identifier::bad_cast:
			/* exception->bad_cast
			 An exception of this type is thrown when a dynamic_cast to a reference type fails
			 the run-time check (e.g. because the types are not related by inheritance), and also
			 from std::use_facet if the requested facet does not exist in the locale.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_cast);
		  break;
			case ExceptionType::Identifier::bad_any_cast:
			/* exception->bad_cast->bad_any_cast
			 Introduced in C++17
			 Defines a type of object to be thrown by the value-returning forms of std::any_cast on failure.
			*/
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_any_cast);
			break;
		  case ExceptionType::Identifier::bad_weak_ptr:
			/* exception->bad_weak_ptr
			 Introduced in C++11
			 std::bad_weak_ptr is the type of the object thrown as exceptions by the constructors
			 of std::shared_ptr that take std::weak_ptr as the argument, when the std::weak_ptr refers to an already deleted object.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_weak_ptr);
		  break;
		  case ExceptionType::Identifier::bad_function_call:
			/* exception->bad_function_call
			  Introduced in C++11
			  std::bad_function_call is the type of the exception thrown by
			  std::function::operator() if the function wrapper has no target.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_function_call);
		  break;
		  case ExceptionType::Identifier::bad_alloc:
			/* exception->bad_alloc
			 std::bad_alloc is the type of the object thrown as exceptions by the allocation
			 functions to report failure to allocate storage. This can be thrown by new.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_alloc);
		  break;
			case ExceptionType::Identifier::bad_array_new_length:
			  /* exception->bad_alloc->bad_array_new_length
			   Introduced in C++11.
			   std::bad_array_new_length is the type of the object thrown as exceptions by
			   the new-expressions to report invalid array lengths if
			   1) array length is negative
			   2) total size of the new array would exceed implementation-defined maximum value
			   3) the number of initializer-clauses exceeds the number of elements to initialize
			   Only the first array dimension may generate this exception; dimensions other
			   than the first are constant expressions and are checked at compile time.
			  */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_array_new_length);
			break;
		  case ExceptionType::Identifier::bad_exception:
			/* exception->bad_exception
			 std::bad_exception is the type of the exception thrown by the C++ runtime in the following situations:
			 1. If std::exception_ptr stores a copy of the caught exception and if the copy constructor of the exception
			 object caught by std::current_exception throws an exception, the captured exception
			 is an instance of std::bad_exception.(C++11).
			 2. If a dynamic exception specification is violated and std::unexpected throws or rethrows an exception
			 that still violates the exception specification, but the exception specification allows
			 std::bad_exception, std::bad_exception is thrown. (C++17)
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_exception);
		  break;
		  case ExceptionType::Identifier::logic_failure:
			/* exception->logic_failure
			 An exception that theoretically can be detected by reading the code.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::logic_failure);
		  break;
		  case ExceptionType::Identifier::bad_variant_access:
		  /* exception->bad_variant_access
		   Introduced in C++17.
		   std::bad_variant_access is the type of the exception thrown in the following situations:
		   1. std::get(std::variant) called with an index or type that does not match the currently active alternative
		   2. std::visit called to visit a variant that is valueless_by_exception
		  */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::bad_variant_access);
		  break;
		  case ExceptionType::Identifier::exception:
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::exception);
		  break;
		  case ExceptionType::Identifier::ParseError:
			/*
			 This exception is thrown if there is any parsing issue in a formula, splitting a string or json string.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::ParseError);
		  break;
		case ExceptionType::Identifier::CancellationException:
			/*
			 This exception indicates that the request is interrupted by the client through a cancellation
			 command. When this exception is thrown the current running request will be cancelled.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::CancellationException);
		  break;
		case ExceptionType::Identifier::NumericalException:
			/*
			 The purpose of this exception to indicate an error while running numerical calculations.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::NumericalException);
		  break;
		case ExceptionType::Identifier::FCSReadingException:
			/*
			 This exception indicates an error while reading FCS files.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::FCSReadingException);
		  break;
		case ExceptionType::Identifier::F1Exception:
			/*
			 This exception indicates that something is wrong in the analysis layer. This
			 exception is mostly thrown in the ParseJSON, and Process method of the requests
			 in the Analysis Layer.
			 */
			logStatement = ExceptionType::ToString(ExceptionType::Identifier::F1Exception);
		  break;
		case ExceptionType::Identifier::disallowed_use_case:
			/*
			 This exception indicates that the request is for a use case that is not considered valid and
			 will not be allowed to be performed.
			 */
		  logStatement = ExceptionType::ToString(ExceptionType::Identifier::disallowed_use_case);
		break;
		case ExceptionType::Identifier::AnalysisAlreadyExists:
			/*
			 When we instruct the engine to use an analysis uuid to create a new analysis or workbench,
			 this exception is thrown to show that the analysis uuid has an analysis already attached to it.
			 */
		  logStatement = ExceptionType::ToString(ExceptionType::Identifier::AnalysisAlreadyExists);
		break;
		case ExceptionType::Identifier::FormulaError:
			/*
			 While running any formula, If engine will face any problem, this exception will be thrown. This
			 type will help us to distinguish any formula processing error from all other types of errors.
			 */
		  logStatement = ExceptionType::ToString(ExceptionType::Identifier::FormulaError);
		break;
		case ExceptionType::Identifier::object_missing:
			/*
			 This exception is thrown if an object is missing from an XML or a JSON. This exception is
			 generic to be used for C++ objects if those are null or missing.
			 */
		  logStatement = ExceptionType::ToString(ExceptionType::Identifier::object_missing);
		break;
		case ExceptionType::Identifier::too_few_events:
			/*
			 If the number of events are not according to a threshold of an operation, this exception is thrown.
			 */
		  logStatement = ExceptionType::ToString(ExceptionType::Identifier::too_few_events);
		break;
		case ExceptionType::Identifier::unexpected_modification:
			/*
			 If a modification type is recognized which is not expected in propagation of the
			 modification, we throw this type of exception.
			 */
		  logStatement = ExceptionType::ToString(ExceptionType::Identifier::unexpected_modification);
		break;
		//In case we don't know which exception
		case ExceptionType::Identifier::catch_all:
		  logStatement = ExceptionType::ToString(ExceptionType::Identifier::catch_all);
		break;
		  default:
			logStatement = "UNKNOWN ERROR"; //shouldn't be possible
		}
	logStatement.append(": ");
	logStatement.append(what());
	if(!fWhere.empty())
	{
		logStatement.append(" location: ");
		logStatement.append(fWhere);
	}
	LOG(ERROR)<<logStatement;
	return fWhere.empty() ? returnMessage : returnMessage+" "+fWhere;
}

EngineException EngineException :: AsEngineException(const std::exception& currentException, std::string where, short int clientErrorCode)
{
	ExceptionType::Identifier identifier = ExceptionType::Identifier::exception;
	try
	{
		std::string exceptionName = typeid(currentException).name(); //Platform dependent implementation of the name() method
		if(const EngineException* e = dynamic_cast<const EngineException*>(&currentException))
			return *e;
		//Now give this name to the ExceptionIdentifier enum and identify the exception to create an ExngineException
		if(!exceptionName.empty())
			identifier = ExceptionType::FromString(exceptionName);
	}
	catch(std::bad_cast& badCastException)
	{
		//It means that the exception maybe not part of the hierarchy.
	}
	catch(std::bad_typeid& badTypeIdException)
	{
		//It means we are trying to apply typeid on a dereferenced pointer
	}
	
	EngineException engineException(identifier, currentException.what(), where, clientErrorCode);
	return engineException;
}
#include <iostream>
#include <sstream.h>
namespace StackTrace
{
	#ifdef WINDOWS
		#include "DbgHelp.h" //The functions in this library are single threaded. Call from more than one thread to the functions will cause undefined behavior.

		#pragma comment(lib, "DbgHelp.lib") //Instruction to linker to load this library. Compiler will ignore this line.

		struct StackFrame
		{
			#ifdef _WIN64
				DWORD64 address;
			#else
				DWORD address;
			#endif
			std::string name;
			std::string module;
			unsigned int line;
			std::string file;
		};

		static std::string basename(const std::string& file)
		{
			unsigned int i = file.find_last_of("\\/");
			if (i == std::string::npos)
			{
				return file;
			}
			else
			{
				return file.substr(i + 1);
			}
		}

		std::string GetStacktrace()
		{
			/*
			To find out more detail about the functions used in this method, follow the link below:
			https://docs.microsoft.com/en-us/windows/win32/api/dbghelp/
			*/
			HANDLE process = GetCurrentProcess();
			HANDLE thread = GetCurrentThread();
			/*
			  CONTEXT Contains processor-specific register data. The system uses CONTEXT structures to perform various internal operations.
			  Refer to the header file WinNT.h for definitions of this structure for each processor architecture.
			*/
			CONTEXT context = {};
			context.ContextFlags = CONTEXT_FULL;
			RtlCaptureContext(&context); //The RtlCaptureContext function retrieves a context record in the context of the caller
			SymInitialize(process, NULL, TRUE);  //The SymInitialize function is used to initialize the symbol handler for a process.
			SymSetOptions(SYMOPT_LOAD_LINES);//The options value can be changed any number of times while the library is in use by an application. The option change affects all future calls to the symbol handler.
			STACKFRAME frame = {};
			#if defined(_WIN64)
				DWORD machine_type = IMAGE_FILE_MACHINE_AMD64;
				frame.AddrPC.Offset = context.Rip; //the rip points at the time of evaluation to the beginning of next instruction.
				frame.AddrPC.Mode = AddrModeFlat; //The mode is the addressing mode. Possible value are 0 means AddrMode1616(16:16 addressing), 1 means AddrMode1632(16:32 addressing), 2 means AddrModeReal(Real-mode addressing), and 3 means AddrModeFlat.
				frame.AddrFrame.Offset = context.Rbp; //rbp is the base pointer, which points to the base of the current stack frame.		frame.AddrFrame.Mode = AddrModeFlat;
				frame.AddrStack.Offset = context.Rsp; //and rsp is the stack pointer, which points to the top of the current stack frame.
				frame.AddrStack.Mode = AddrModeFlat; //Flat addressing. This is the only addressing mode supported by the library.
			#else
				DWORD machine_type = IMAGE_FILE_MACHINE_I386;
				frame.AddrPC.Offset = context.Eip; //a variant of rip in 32b mode
				frame.AddrPC.Mode = AddrModeFlat;
				frame.AddrFrame.Offset = context.Ebp; //a variant of rbp in 32b mode
				frame.AddrFrame.Mode = AddrModeFlat;
				frame.AddrStack.Offset = context.Esp; //a variant of rsp in 32b mode
				frame.AddrStack.Mode = AddrModeFlat;
			#endif
			bool first = true;

			std::vector<StackFrame> stackFramesVector;
			//For StackWalk, see https://docs.microsoft.com/en-us/windows/win32/api/dbghelp/nf-dbghelp-stackwalk
			//For StackWalk64, see https://docs.microsoft.com/en-us/windows/win32/api/dbghelp/nf-dbghelp-stackwalk64
			while (StackWalk64(machine_type, process, thread, &frame, &context, NULL, SymFunctionTableAccess, SymGetModuleBase, NULL))
			{
				StackFrame stackFrame = {};
				stackFrame.address = frame.AddrPC.Offset;

				#if _WIN64
					DWORD64 moduleBase = 0;
				#else
					DWORD moduleBase = 0;
				#endif
				// For SymGetModuleBase64, see https://docs.microsoft.com/en-us/windows/win32/api/dbghelp/nf-dbghelp-symgetmodulebase64
				// For SymGetModuleBase64, see https://docs.microsoft.com/en-us/windows/win32/api/dbghelp/nf-dbghelp-symgetmodulebase
				moduleBase = SymGetModuleBase64(process, frame.AddrPC.Offset); //Retrieves the base address of the module that contains the specified address.

				char moduleBuff[MAX_PATH];
				/*
					GetModuleFileNameA retrieves the fully qualified path for the file that contains the specified module.
					The module must have been loaded by the current process. For more detail, see https://docs.microsoft.com/en-us/windows/win32/api/libloaderapi/nf-libloaderapi-getmodulefilenamea
					*/
				stackFrame.module = "Unknown Module";
				if (moduleBase && GetModuleFileNameA((HINSTANCE)moduleBase, moduleBuff, MAX_PATH))
				{
					stackFrame.module = basename(moduleBuff);
				}
				#if _WIN64
					DWORD64 offset = 0;
				#else
					DWORD offset = 0;
				#endif
				char symbolBuffer[sizeof(IMAGEHLP_SYMBOL) + 255];
				PIMAGEHLP_SYMBOL symbol = (PIMAGEHLP_SYMBOL)symbolBuffer;
				symbol->SizeOfStruct = (sizeof IMAGEHLP_SYMBOL) + 255;
				symbol->MaxNameLength = 254;
				/*
					SymGetSymFromAddr and SymGetSymFromAddr locates the symbol for the specified address.
					For more details, see
					https://docs.microsoft.com/en-us/windows/win32/api/dbghelp/nf-dbghelp-symgetsymfromaddr64 and
					https://docs.microsoft.com/en-us/windows/win32/api/dbghelp/nf-dbghelp-symgetsymfromaddr
				*/
				stackFrame.name = "Unknown Function";
				if (SymGetSymFromAddr64(process, frame.AddrPC.Offset, &offset, symbol))
				{
					stackFrame.name = symbol->Name;
				}
				IMAGEHLP_LINE line;
				line.SizeOfStruct = sizeof(IMAGEHLP_LINE);

				DWORD offset_ln = 0;
				stackFrame.line = 0;
				if (SymGetLineFromAddr64(process, frame.AddrPC.Offset, &offset_ln, &line))
				{
					stackFrame.file = line.FileName;
					stackFrame.line = line.LineNumber;
				}
				if (!first)
				{
					stackFramesVector.push_back(stackFrame);
				}
				first = false;
			}
			SymCleanup(process); //This function deallocates all resources associated with the process handle.
			std::stringstream organizedStack;
			organizedStack << "\n" << "\t\t" << "======= Backtrace: =========" << "\n";
			int stackFrameLineNumber = 1;
			for (StackFrame singleFrame : stackFramesVector)
			{
				organizedStack << stackFrameLineNumber <<"\t"<<singleFrame.module<<"\t\t"<<"0x" << std::hex << singleFrame.address;
				organizedStack <<"\t"<<singleFrame.name<<"\t"<<singleFrame.file<<": "<<singleFrame.line<<endl;
				stackFrameLineNumber++;
			}
			return organizedStack.str();
		}

	#else //Not WINDOWS

		#include <cxxabi.h>
		#include <execinfo.h>
		using namespace std;
		/*
		 Get Stack trace method is copied (from src/enginesrc/includes/stacktrace.h), renamed and modified according to our needs for
		 Signal handling. This is Mac Only stacktrace.
		 */
			string GetStacktrace()
			{
				unsigned int max_frames = 63;
				stringstream stackTrace;
				stackTrace << "\n" << "\t\t======= Backtrace: =========" << "\n";

					// storage array for stack trace address data
					void* addrlist[max_frames + 1];

				// retrieve current stack addresses
				int addrlen = backtrace(addrlist, sizeof(addrlist) / sizeof(void*));

				if (addrlen == 0)
				{
					stackTrace << "  <empty, possibly corrupt> " << ::std::endl;
					return stackTrace.str();
				}

				// resolve addresses into strings containing "filename(function+address)",
				// this array must be free()-ed
				char** symbollist = backtrace_symbols(addrlist, addrlen);

				// allocate string which will be filled with the demangled function name
				size_t funcnamesize = 256;
				char* funcname = (char*)malloc(funcnamesize);

				// iterate over the returned symbol lines. skip the first, it is the
				// address of this function.
				for (int i = 1; i < addrlen; i++)
				{
					char* begin_name = 0, * begin_offset = 0, * end_offset = 0;

					// find parentheses and +address offset surrounding the mangled name:
					// ./module(function+0x15c) [0x8048a6d]
					for (char* p = symbollist[i]; *p; ++p)
					{
						if (*p == '(')
							begin_name = p;
						else if (*p == '+')
							begin_offset = p;
						else if (*p == ')' && begin_offset) {
							end_offset = p;
							break;
						}
					}

					if (begin_name && begin_offset && end_offset
						&& begin_name < begin_offset)
					{
						*begin_name++ = '\0';
						*begin_offset++ = '\0';
						*end_offset = '\0';

						// mangled name is now in [begin_name, begin_offset) and caller
						// offset in [begin_offset, end_offset). now apply
						// __cxa_demangle():

						int status;
						char* ret = abi::__cxa_demangle(begin_name,
							funcname, &funcnamesize, &status);
						if (status == 0) {
							funcname = ret; // use possibly realloc()-ed string
							stackTrace << " " << symbollist[i] << " : " << funcname << "+" << begin_offset << endl;
						}
						else {
							// demangling failed. Output function name as a C function with
							// no arguments.
							stackTrace << " " << symbollist[i] << " : " << begin_name << "+" << begin_offset << endl;
						}
					}
					else
					{
						// couldn't parse the line? report the whole line.
						stackTrace << "  " << symbollist[i] << ::std::endl;
					}
				}
				free(funcname);
				free(symbollist);
				return stackTrace.str();
			}
	#endif //end #ifdef WINDOWS
} //End namespace StackTrace
