/*
 * Exceptions.h
 *
 *  Created on: Feb 6, 2018
 *      Author: rhalpert
 */

#ifndef SRC_ENGINESRC_ERROR_EXCEPTIONS_H_
#define SRC_ENGINESRC_ERROR_EXCEPTIONS_H_

#include <string>


#define Error_EXCEPTION_DURING_PROCESSING 406 //This error code is defined in the GlobalTypes.h, however, we can't import the header because of some header conflicts on Window OS.

class ExceptionType
{
	public:
	enum Identifier{logical_error, domain_error, invalid_argument, length_error, out_of_range,
		overflow_error, range_error, underflow_error, bad_alloc,
		bad_cast, bad_typeid, bad_exception, logic_failure, runtime_error,
		exception, future_error, non_existent_local_time, ambiguous_local_time,
		bad_any_cast, bad_array_new_length, bad_variant_access, system_error, failure, format_error,
		filesystem_error, bad_optional_access, regex_error, tx_exception, bad_weak_ptr, bad_function_call,
		F1Exception, ParseError, CancellationException, NumericalException,
		FCSReadingException, catch_all, disallowed_use_case, AnalysisAlreadyExists, FormulaError, object_missing, unexpected_modification, too_few_events};
	static ExceptionType::Identifier FromString(std::string identifier);
	static std::string ToString(Identifier identifier);
};

class EngineException : public std::exception
{
public:
	EngineException(ExceptionType::Identifier identifier, std::string what, std::string where = std::string(), short int clientErrorCode = Error_EXCEPTION_DURING_PROCESSING) noexcept : fIdentifier(identifier),fWhat(what), fWhere(where), fClientErrorCode(clientErrorCode)  {}
	EngineException(const EngineException& e) noexcept : fIdentifier(e.fIdentifier), fWhat(e.fWhat), fWhere(e.fWhere), fClientErrorCode(e.fClientErrorCode)  {}
	virtual ~EngineException(){}
private:
	ExceptionType::Identifier fIdentifier;
	std::string fWhat;
	std::string fWhere;
	short int fClientErrorCode;
public:
	virtual const char * what () const noexcept {	return fWhat.c_str(); }
	const short int GetClientErrorCode() const noexcept {	return fClientErrorCode; }
	const ExceptionType::Identifier GetExceptionIdentifier() const { return fIdentifier;}
	const std::string exceptionDetails() const;
	const std::string where() const { return fWhere;}
	static EngineException AsEngineException(const std::exception& currentException, std::string where = std::string(), short int clientErrorCode = Error_EXCEPTION_DURING_PROCESSING);
};

namespace StackTrace
{
	std::string GetStacktrace();
}
#endif /* SRC_ENGINESRC_ERROR_EXCEPTIONS_H_ */
