#ifndef NAME_VALUE_HPP
#define NAME_VALUE_HPP

#include <string>
#include <variant>

namespace FJ
{
   class NameValue
   {
   public:
      NameValue( const std::string& name,bool value ) : m_name(name),m_value(value){};
      NameValue( const std::string& name,double value ) : m_name(name),m_value(value){};
      NameValue( const std::string& name,int value ) : m_name(name),m_value(value){};
      NameValue( const std::string& name,std::string value ) : m_name(name),m_value(value){};
      NameValue( const std::string& name,const char* value ) : NameValue( name,value ? std::string( value ) : std::string() ) {};

      NameValue() = default;
      NameValue( const NameValue& source ) = default;
      NameValue& operator=( const NameValue& rhs ) = default;
      NameValue( NameValue&& source ) = default;
      NameValue& operator=( NameValue&& rhs ) = default;

      std::string name() const { return m_name; };
      bool getBool() const { return std::get<bool>( m_value ); }
      double getDouble() const { return std::get<double>( m_value ); }
      int getInt() const { return std::get<int>( m_value ); }
      std::string getString() const { return std::get<std::string>( m_value ); }
      // A get*() method produces a result only if the corresponding is*() method returns true.
      // Otherwise, it throws std::bad_variant_access.

      bool isBool() const { return m_value.index()==0; };
      bool isDouble() const { return m_value.index()==1; };
      bool isInt() const { return m_value.index()==2; };
      bool isString() const { return m_value.index()==3; };

   private:
      std::string m_name;
      std::variant<bool,double,int,std::string> m_value;
   };
} // end namespace FJ
#endif