#include "Summary.hpp"
#include "Settings.hpp"
#include <fstream>

using namespace FJ;
using namespace std;

FJ::Summary::Summary( bool success, int numEvents,
   const std::vector<NameValue>& calculations, 
   const std::string& information )
   : mSuccess(success), mNumEvents(numEvents),
   mInformation(information), mCalculations(calculations)
{}

void FJ::Summary::add_calculations( const std::vector<NameValue>& keyValue )
{
   mCalculations.insert( mCalculations.end(), keyValue.cbegin(), keyValue.cend() );
}

void FJ::Summary::add_calculation( const NameValue& keyValue )
{
   mCalculations.push_back( keyValue );
}

void FJ::Summary::clear_calculations()
{
   mCalculations.clear();
}

bool FJ::Summary::has_calculations() const
{
   return !mCalculations.empty();
}

bool FJ::Summary::has_information() const
{
   return !mInformation.empty();
}

std::string FJ::Summary::information() const
{
   return mInformation;
}

void FJ::Summary::information( const std::string& info )
{
   mInformation = info;
}

bool FJ::Summary::is_success() const
{
   return mSuccess;
}

void FJ::Summary::is_success( bool success )
{
   mSuccess = success;
}

int FJ::Summary::numEvents() const
{
   return mNumEvents;
}

void Summary::saveAsINI( const std::string& fileName ) const
{
   static const char* separator { " = " };

   ofstream out( fileName );

   if( !out )
      throw runtime_error("Couldn\'t open summary file "+fileName); 

   out << status_Att << separator << (is_success()?success_Val:failure_Val) << '\n';
   if( has_information() )
      out << information_Att << separator << information() << '\n';
}


