#ifndef SUMMARY_HPP
#define SUMMARY_HPP

#include "NameValue.hpp"

#include <string>
#include <vector>

namespace FJ
{
   class Summary
   {
   public:

      Summary( bool successful, int numEvents,
         const std::vector<NameValue>& calculations=std::vector<NameValue>(),
         const std::string& information = std::string() );

      Summary() = default;
      Summary( const Summary& source ) = default;
      Summary& operator=( const Summary& rhs ) = default;

      Summary( Summary&& source ) = default;
      Summary& operator=( Summary&& rhs ) = default;

      void add_calculations( const std::vector<NameValue>& results );
      void add_calculation( const NameValue& result );
      void clear_calculations();

      bool has_calculations() const;
      bool has_information() const;

      std::string information() const;
      void information( const std::string& info );

      bool is_success() const;
      void is_success( bool success );

      int numEvents() const;

      void saveAsINI( const std::string& fileName ) const;

   private:

      inline static const char* failure_Val { "failure" };
      inline static const char* information_Att { "information" };
      inline static const char* status_Att { "status" };
      inline static const char* success_Val { "success" };

      //void store( const std::vector<std::string>& inputFiles,
      //   const std::vector<NameValue>& Summary, const std::vector<std::string>& outputFiles,
      //   const std::vector<std::string>& calculation_names, bool success=false,
      //   const std::string& information=std::string() );

      std::vector<NameValue> mCalculations;

      std::string mInformation;

      int mNumEvents { 0 };

      bool mSuccess { false };
   };

} // end namespace FJ
#endif