#ifndef CSV_HPP
#define CSV_HPP

#include <fstream>
#include <string>
#include <vector>
namespace FJ
{

bool read_csv( const std::string& file,
	std::vector< std::vector<std::string> >& lines, bool skip_first_line=false );
// assumes delimiter is a comma

// T must work with the insertion operator, i.e.,  out << T
// delimiter can be something other than a comma, e.g., a tab
template<class T>
bool write_csv( const std::string& file,
	const std::vector< std::vector<T> >& lines,
	const std::vector<std::string>& header, char delimiter = ',' )
{
	auto len = lines.size();
	std::ofstream out( file.c_str() );
	if( !out )
		return false;

	if( !header.empty() )
	{
		// write all but the last element in the header followed by the delimiter
		for( std::vector<std::string>::const_iterator j = header.begin();
			j != header.end()-1; j++ )
			out << *j << delimiter;

		// write the last element without a delimiter after it
		out << header.back() << std::endl;
	}

	for( typename std::vector< std::vector<T> >::const_iterator i = lines.begin();
		i != lines.end(); ++i )
	{
		// write all but the last element in the line followed by the delimiter
		for( typename std::vector<T>::const_iterator j = i->begin(); j != i->end()-1; j++ )
		{
			out << *j << delimiter;
		}

		// write the last element without a delimiter after it
		out << i->back() << std::endl;
	}

	// out converted to bool
	return out.good();
}

// convenience version for writing without a header
// T must work with the insertion operator, i.e.,  out << T
template<class T>
bool write_csv( const std::string& file,
	const std::vector< std::vector<T> >& lines,
	char delimiter = ',' )
{
	std::vector<std::string> dummy;
	return FJ::write_csv<T>( file, lines, dummy, delimiter );
}


} // end namespace FJ

#endif