#ifndef FLOWJO_EXCEPTIONS_HPP
#define FLOWJO_EXCEPTIONS_HPP

#include "FlowJo_Exception.hpp"

namespace FJ
{
   /*
   * These are classes that represent general types of exceptions, i.e., those that can
   * occur in any project. Having these makes it so that users don't have to write similar
   * ones.
   * The classes are:
   * Bad_Enum_Int - can't convert an integer to an enum
   * Bad_Enum_Text - can't convert text to an enum
   * Bad_File_Open - can't open a file
   * Bad_XML_Read - can't read (parse) an XML file
   * 
   * Created by Greg Reese on February 15, 2022
   */

   class Bad_Enum_Int : public FlowJo_Exception
   {
   public:
      Bad_Enum_Int( const char* text = "Integer is not an enum value" )
         : FlowJo_Exception( text ) {};
      Bad_Enum_Int( int num, const std::string& enum_name );
      // stores message of "num is not a valid value of the enum y"
   };

   class Bad_Enum_Text : public FlowJo_Exception
   {
   public:
      Bad_Enum_Text( const char* text = "Text does not represent an enum" )
         : FlowJo_Exception( text ) {};
      Bad_Enum_Text( const std::string& bad_text,
         const std::string& enum_name );
      // stores message of "bad_text is not a text version of the enum y"
   };

   class Bad_File_Open : public FlowJo_Exception
   {
   public:
      Bad_File_Open() : Bad_File_Open( "Couldn\'t open file", false ) {};
      Bad_File_Open( const std::string& text, bool append=true );
   };

   class Bad_File_Read : public FlowJo_Exception
   {
   public:
      Bad_File_Read(const char* text = "Couldn\'t read file") : FlowJo_Exception(text) {};
      // successfully opened file but couldn't read it
   };

   class Bad_XML_Read : public FlowJo_Exception
   {
   public:
      Bad_XML_Read( const char* text = nullptr );
      // if text is nullptr, stores "Can\'t read XML", else stores text

      Bad_XML_Read( int num );
      // stores "Can\'t read XML. Error: num"

      Bad_XML_Read( const std::string& text, int num,
         bool add_space=true );
      // stores text with num at end. If add_space is true,
      // adds one space between text and num
   };
} // end of namespace V9To11

#endif

